<?php
use Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType;

final class WC_Apppago_Gateway_Blocks_Support extends AbstractPaymentMethodType
{

    private $gateway;

    protected $name = 'apppago_gateway'; // payment gateway id

    public function initialize() {
        // get payment gateway settings
        $this->settings = get_option( "woocommerce_{$this->name}_settings", array() );

        // you can also initialize your payment gateway here
        $gateways = WC()->payment_gateways->payment_gateways();
        $this->gateway  = $gateways[ $this->name ];
    }

    public function is_active() {
        return ! empty( $this->settings[ 'enabled' ] ) && 'yes' === $this->settings[ 'enabled' ];
    }

    public function get_payment_method_script_handles() {

        wp_register_script(
            'wc-apppago-blocks-integration',
            plugin_dir_url( __FILE__ ) . 'apppago-block/build/index.js',
            array(
                'wc-blocks-registry',
                'wc-settings',
                'wp-element',
                'wp-html-entities',
            ),
            null,
            true
        );

        return array( 'wc-apppago-blocks-integration' );

    }

    public function get_payment_method_data() {
        return array(
//        $this->icon = add_filter('woocommerce_gateway_icon', 'custom_gateway_icon', 10, 2);
//        $this->has_fields = true;
//        $this->method_title = 'AppPago';
//        $this->method_description = 'Paga in comode quote con AppPago';
//        $this->supports = array(
//            'products'
//        );
            'title'        => $this->gateway->method_title,
            // almost the same way:
            'description'  => $this->gateway->method_description,
            // if $this->gateway was initialized on line 15
            'supports'  => array_filter( $this->gateway->supports, [ $this->gateway, 'supports' ] ),
            'placeOrderButtonLabel' => __( 'Procedi su AppPago', 'woocommerce-gateway-apppago' ),
            'content' => $this->gateway->payment_fields(true),
            'icon'         => plugin_dir_url( __FILE__ ) . 'apppago.webp',
            // example of getting a public key
            // 'publicKey' => $this->get_publishable_key(),
        );
    }

    //private function get_publishable_key() {
    //	$test_mode   = ( ! empty( $this->settings[ 'testmode' ] ) && 'yes' === $this->settings[ 'testmode' ] );
    //	$setting_key = $test_mode ? 'test_publishable_key' : 'publishable_key';
    //	return ! empty( $this->settings[ $setting_key ] ) ? $this->settings[ $setting_key ] : '';
    //}

}